"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const tl = require("vsts-task-lib/task");
const Q = require("q");
const rm = require("typed-rest-client/RestClient");
let proxyUrl = tl.getVariable("agent.proxyurl");
var requestOptions = proxyUrl ? {
    proxy: {
        proxyUrl: proxyUrl,
        proxyUsername: tl.getVariable("agent.proxyusername"),
        proxyPassword: tl.getVariable("agent.proxypassword"),
        proxyBypassHosts: tl.getVariable("agent.proxybypasslist") ? JSON.parse(tl.getVariable("agent.proxybypasslist")) : null
    }
} : {};
let ignoreSslErrors = tl.getVariable("VSTS_ARM_REST_IGNORE_SSL_ERRORS");
requestOptions.ignoreSslError = ignoreSslErrors && ignoreSslErrors.toLowerCase() == "true";
let rc = new rm.RestClient(tl.getVariable("AZURE_HTTP_USER_AGENT"), null, null, requestOptions);
var azureStackEnvironment = "AzureStack";
var defaultAuthorityUrl = "https://login.windows.net/";
function initializeAzureRMEndpointData(connectedServiceName) {
    return __awaiter(this, void 0, void 0, function* () {
        var endPoint = new Array();
        endPoint["servicePrincipalClientID"] = tl.getEndpointAuthorizationParameter(connectedServiceName, 'serviceprincipalid', false);
        endPoint["servicePrincipalKey"] = tl.getEndpointAuthorizationParameter(connectedServiceName, 'serviceprincipalkey', false);
        endPoint["tenantID"] = tl.getEndpointAuthorizationParameter(connectedServiceName, 'tenantid', false);
        endPoint["subscriptionId"] = tl.getEndpointDataParameter(connectedServiceName, 'subscriptionid', true);
        endPoint["envAuthUrl"] = tl.getEndpointDataParameter(connectedServiceName, 'environmentAuthorityUrl', true);
        endPoint["url"] = tl.getEndpointUrl(connectedServiceName, true);
        endPoint["environment"] = tl.getEndpointDataParameter(connectedServiceName, 'environment', true);
        endPoint["activeDirectoryResourceId"] = tl.getEndpointDataParameter(connectedServiceName, 'activeDirectoryServiceEndpointResourceId', true);
        if (endPoint["environment"] != null && endPoint["environment"].toLowerCase() == azureStackEnvironment.toLowerCase()) {
            if (!endPoint["envAuthUrl"] || !endPoint["activeDirectoryResourceId"]) {
                endPoint = yield initializeAzureStackData(endPoint);
                endPoint["envAuthUrl"] = endPoint['environmentAuthorityUrl'];
                endPoint["activeDirectoryResourceId"] = endPoint['activeDirectoryServiceEndpointResourceId'];
                if (endPoint["envAuthUrl"] == null) {
                    throw tl.loc("UnableToFetchAuthorityURL");
                }
                if (endPoint["activeDirectoryResourceId"] == null) {
                    throw tl.loc("UnableToFetchActiveDirectory");
                }
            }
        }
        else {
            endPoint["envAuthUrl"] = (endPoint["envAuthUrl"] != null) ? endPoint["envAuthUrl"] : defaultAuthorityUrl;
            endPoint["activeDirectoryResourceId"] = endPoint["url"];
        }
        return endPoint;
    });
}
exports.initializeAzureRMEndpointData = initializeAzureRMEndpointData;
function initializeAzureStackData(endpoint) {
    return __awaiter(this, void 0, void 0, function* () {
        var deferred = Q.defer();
        var headers = {
            'Content-Type': 'application/json'
        };
        var azureStackDependencyDataUrl = endpoint.url + "/metadata/endpoints?api-version=2015-01-01";
        let options = {};
        options.additionalHeaders = headers;
        let promise = rc.get(azureStackDependencyDataUrl, options);
        promise.then((response) => {
            if (response.statusCode === 200) {
                let result = response.result;
                var authenticationData = result.authentication;
                if (authenticationData) {
                    var loginEndpoint = authenticationData.loginEndpoint;
                    if (loginEndpoint) {
                        loginEndpoint += (loginEndpoint.lastIndexOf("/") == loginEndpoint.length - 1) ? "" : "/";
                        endpoint['activeDirectoryAuthority'] = loginEndpoint;
                        endpoint['environmentAuthorityUrl'] = loginEndpoint;
                    }
                    var audiences = authenticationData.audiences;
                    if (audiences) {
                        if (audiences.length > 0) {
                            endpoint['activeDirectoryServiceEndpointResourceId'] = audiences[0];
                        }
                    }
                }
                if (result.graphEndpoint) {
                    endpoint['graphUrl'] = result.graphEndpoint;
                }
                if (result.galleryUrl) {
                    endpoint['galleryUrl'] = result.galleryUrl;
                }
                if (result.portalEndpoint) {
                    endpoint['portalEndpoint'] = result.portalEndpoint;
                }
                var endpointUrl = endpoint.url;
                endpointUrl += (endpointUrl.lastIndexOf("/") == endpointUrl.length - 1) ? "" : "/";
                var domain = "";
                try {
                    var index = endpointUrl.indexOf('.');
                    domain = endpointUrl.substring(index + 1);
                    domain = (domain.lastIndexOf("/") == domain.length - 1) ? domain.substring(0, domain.length - 1) : domain;
                }
                catch (error) {
                    deferred.reject(tl.loc("SpecifiedAzureRmEndpointIsInvalid", endpointUrl));
                }
                endpoint['AzureKeyVaultDnsSuffix'] = ("vault" + domain).toLowerCase();
                endpoint['AzureKeyVaultServiceEndpointResourceId'] = ("https://vault." + domain).toLowerCase();
                deferred.resolve(endpoint);
            }
            else {
                tl.debug("Action: initializeAzureStackData, Response: " + JSON.stringify(response));
                deferred.reject(tl.loc("FailedToFetchAzureStackDependencyData", response.statusCode));
            }
        }, (error) => {
            deferred.reject(error);
        });
        return deferred.promise;
    });
}
exports.initializeAzureStackData = initializeAzureStackData;
